<?xml version="1.0" encoding="UTF-8"?>

<#import "common_module_traversal_utils.ftl" as utils>
<#import "macros_common_general.ftl" as com>
<#import "common_module_front_page_docbook.ftl" as fp>

<#-- Initialize common variables -->
<@com.initializeMainVariables/>

<#-- Relevance for EU PPP context: in order to add PD claims -->
<#if _subject.submissionType?has_content && _subject.submissionType?matches("EU_PPP_.*")>
    <@com.initiRelevanceForPPP relevance/>
</#if>

<#assign locale = "en" />
<#assign sysDateTime = .now>

<#-- Root entity -->
<#assign rootEntity = entity.root>

<#-- Define the main macro to be used to retrieve output by the traversal approach -->
<#assign mainMacroName = "printIfDataProtectionElementNode"/>
<#assign mainMacroArgs = ['docElementNode', 'sectionNode', 'entityDocument']/>

<#-- Make global the macroCall as well as each macro used in it -->
<#global mainMacroCall="<@" + mainMacroName + " " + mainMacroArgs?join(" ") + "/>"/>
<#global printIfDataProtectionElementNode = printIfDataProtectionElementNode/>
<#global printDataProtectionField = printDataProtectionField/>
<#global tableOutput=tableOutput/>

<#-- Counter of claims -->
<#assign index=0/>

<book version="5.0" xmlns="http://docbook.org/ns/docbook" xmlns:xi="http://www.w3.org/2001/XInclude">

    <#-- Get front page -->
    <@fp.getFrontPage _subject _dossierHeader "List of Confidentiality Claims"/>

    <#-- Chapters -->
    <chapter label="1">
        <title role="HEAD-1">List of Confidentiality Claims</title>

        <para role="small">

            <#-- Table header -->
            <table border="1">
                <title>Confidentiality claims</title>

                <col width="3%" />
                <col width="10%" />
                <col width="15%" />
                <col width="17%" />
                <col width="10%" />
                <col width="40%" />
                <col width="5%" />

                <tbody>
                <tr>
                    <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">#</emphasis></th>
                    <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Entity</emphasis></th>
                    <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">IUCLID section</emphasis></th>
                    <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Document</emphasis></th>
                    <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Field</emphasis></th>
                    <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Claim and Justification</emphasis></th>
                    <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Accept Y/N</emphasis></th>
                </tr>

                <#-- Traversal approach: prints one row per claim -->
                <@utils.traversal rootEntity 8/>

                </tbody>
            </table>
        </para>

    </chapter>

</book>

<#----------------------------------------------->
<#-- Macro and function definitions            -->
<#----------------------------------------------->

<#-- printIfDataProtectionElementNode checks if an element node in a document corresponds
     to a data protection field. If it does, it calls the printDataProtectionField macro.

     Inputs:
     - docElementNode: NODE (element) of the document being traversed (initialy root = the document itself, then all the children)
     - sectionNode: NODE of the table of contents of the entity where the document is found
     - entityDocument: ENTITY the document being traversed belongs to
-->
<#macro printIfDataProtectionElementNode docElementNode sectionNode entityDocument>
    <#if docElementNode?node_type == "data_protection">

        <@printDataProtectionField docElementNode sectionNode entityDocument/>
    </#if>
</#macro>

<#-- printDataProtectionField prints a confidentiality claim as a table row, using the
     tableOutput macro.

     Inputs (correspond to the ones of the previous macro):
     - fieldNode: NODE (element) of the document being traversed (initialy root = the document itself, then all the children)
     - sectionNode: NODE of the table of contents of the entity where the document is found
     - entityDocument: ENTITY the document being traversed belongs to
-->
<#macro printDataProtectionField fieldNode sectionNode entityDocument>

    <#-- Consider claim if:
         - flag is set
         OR
         - justification is provided and remit is PPP (PD claim)
     -->
    <#if fieldNode.confidentiality?has_content ||
        (fieldNode.justification?has_content && pppRelevant??)>

        <#assign index=index+1/>

        <#-- Get section name and number -->
        <#local sectionName = "">
        <#if sectionNode?has_content>
            <#if sectionNode.number?has_content>
                <#local sectionName = sectionNode.number + " ">
            </#if>
            <#local sectionName = sectionName + sectionNode.title>
        </#if>

        <#-- Get index for repeatable block 
            NOTE: unclear if this can be retrieved directly; 
            the only option seems to 1) retrieve the block UUID, 2) list all block and get the index of the itam with same UUID
        -->  
        <#local rowIndex=0/>
        <#local ancestors = fieldNode?ancestors/>
        <#if (ancestors?size>1) && ancestors[1]?node_type=="repeatable">
            <#local blockUUID = ancestors[0].uuid/>

            <#list ancestors[1] as item>
                <#if item.uuid==blockUUID>
                    <#local rowIndex=item?index+1/>
                    <#break>
                </#if>
            </#list>
        </#if>

        <#-- Get the document: the root of the fieldNode is the node that represents the Document to which the field belongs to -->
        <#local sectionDoc = fieldNode?root>
        <#local sectionDocType = sectionDoc.documentType>
        <#if !utils.isEntityDocument(sectionDoc)>
            <#local sectionDocType = sectionDocType + "." + sectionDoc.documentSubType>
        </#if>

        <#-- Get the field path and label -->
        <#local fieldPath = fieldNode._path>
<#--        <@iuclid.label for=fieldNode var="fieldLabel"/> NOTE: since label might not be enough, better to parse the path -->
        <#local fieldLabel = getLabelFromPath(fieldPath, sectionDoc)/>

        <#-- Get the web URL pointing to the document -->
        <#assign docUrl=utils.getDocumentUrl(sectionDoc) />

        <#-- Get the confidentiality claim, legislations and justification -->
        <#local confidentiality = utils.getPhraseCodeValue(fieldNode.confidentiality)>
        <#local confLegislations = utils.getTextForConfidentialityLegislations(fieldNode.legislations)>
        <#assign confJustification><@com.text fieldNode.justification 'literal'/></#assign>


        <@tableOutput entityDocument sectionName sectionDoc docUrl fieldLabel confidentiality confLegislations confJustification rowIndex/>

    </#if>
</#macro>

<#-- tableOutput defines and prints a table row containing the necessary information
     to identify and assess a confidentiality claim

     Inputs:
     - entityDocument: ENTITY the document being traversed belongs to
     - sectionName: STRING containing the number and name of the ToC section where the claim is found
     - sectionDoc: DOCUMENT where the claim is found
     - docUrl: STRING containing the url of the document
     - fieldLabel: STRING containing the field label
     - confidentiality: STRING containing the confidentiality claim type from the picklist e.g. CBI
     - confLegislations: STRING containing the legislations the claim is applicable to e.g. EU PPP
     - confJustification: STRING containing the justification for the claim
     - rowIndex: INT containing number of element in repeatable block
-->
<#macro tableOutput entityDocument sectionName sectionDoc docUrl fieldLabel confidentiality confLegislations confJustification rowIndex>
    <tr>
        <td>${index}</td>

        <#-- IUCLID Entity type and name -->
        <td>
            ${entityDocument.documentType?replace("_", " ")}:
            <?linebreak?>
            <@com.text entityDocument.name/>
        </td>

        <#-- IUCLID section name -->
        <td>
            <@com.text sectionName />
        </td>

        <#-- IUCLID document name wiht url -->
        <td>
            <ulink url="${docUrl}">
                <@com.text sectionDoc.name />
            </ulink>
        </td>

        <#-- IUCLID document field label, extracted from the path -->
        <td>
            ${fieldLabel}
            <#if rowIndex?has_content && rowIndex!=0>
                (${rowIndex})
            </#if>
        </td>

        <#-- Claim and justification -->
        <td>
            <#if confidentiality?has_content><emphasis role="bold">[${confidentiality}]</emphasis></#if>
            ${confJustification}
        </td>

        <#-- Empty field for assessment-->
        <td></td>

    </tr>
</#macro>

<#-- getLabelFromPath extracts a human-readable label from the path of a data protection
     element, by:
     - removing the last element of the path, which usually refers to data protection and it's not informative
     - removing the document type and subtype
     - changing label to "Full document" if flag is at the top of the document (e.g. AdministrativeData)
     - capitalising and separating words in the path

     Inputs:
     - path: absolute path of the element (node)

     Output:
     - label: STRING containing the extracted label
-->
<#function getLabelFromPath fieldPath sectionDoc>
    <#attempt>
        <#local field=fieldPath?split(".")/>

        <#-- Remove document type -->
        <#if field[0] == sectionDoc.documentType>
            <#if (field?size>1)>
                <#local field=field[1..field?size-1]/>
            <#else>
                <#local field=[]/>
            </#if>
        </#if>

        <#-- Remove document subtype -->
        <#if sectionDoc.documentSubType?has_content && field[0] == sectionDoc.documentSubType>
            <#if (field?size>1)>
                <#local field=field[1..field?size-1]/>
            <#else>
                <#local field=[]/>
            </#if>
        </#if>

        <#-- Remove last element (DataProtection) -->
        <#if field[field?size-1]?matches(".*protection", "i")>
            <#if (field?size>1)>
                <#local field=field[0..field?size-2]/>
            <#else>
                <#if field[field?size-1] == "DataProtection">
                    <#local field=[]/>
                <#else>
                    <#local field=[field[0]?replace("protection", "", "i")]/>
                </#if>
            </#if>
        </#if>

        <#local field=field?join(" / ")/>
        <#--        <#local field = field?replace("${sectionDoc.documentType}[ / ]*", "", "r")?replace("${sectionDoc.documentSubType}[ / ]*", "", "r")/>-->

        <#-- Change name-->
        <#if field=="AdministrativeData" || field=="AdministrativeDataSummary" || field=="ConfidentialityInfo" || field="">
            <#local field="Full document"/>
        <#else>
            <#local field=field?replace("([A-Z]{1})", " $1", "r")?lower_case?cap_first/>
        </#if>
    <#recover>
        <#local field = ""/>
    </#attempt>

    <#return field/>
</#function>
